package HyperEps;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import jmetal.core.SolutionSet;
import jmetal.operators.mutation.SwapMutation;

public class Algo {
	 public static double IntersectX(Function f1, Function f2) {
		  return (f2.n - f1.n) / (f1.m - f2.m);
	  }
	 
	 private static double EPSILON = 1e-10;
	 
	 // slow calculation of the maximum of the functions
	 private static double[] CalcMaxSlow(Function[] fun, double X[]) {
		  int N = X.length;
		  
		  double[] best = new double[N];
		  
		  for(int n=N-2; n>=0; n--) {
			  best[n] = 0;
			  for(int m=n+1;m<N; m++) {
				  best[n] = Math.max(best[n], fun[m].evaluate(X[n]));
			  }
		  }
		  
		  return best;
	  }
	 
	 
	 private static int[] CalcMaxPoint(Function[] f, int N, double[] X) {
	      int[] best = new int[N];
	      Function[] s = new Function[N];
	      int[] pos = new int[N];

	      int len = 0;
	      int left = 0;
	      int right = -1;

	      for (int i = N - 2; i >= 0; i--) {
	        while (len > 1 ) {
	          double d1 = IntersectX(f[i+1], s[right]);
	          double d2 = IntersectX(s[right], s[right - 1]);

	          if (d1 > d2 || Math.abs(d1-d2) < EPSILON) {
	            len--;
	            right--;
	          } else {
	            break;
	          }
	        }

	        s[right+1] = f[i+1];
	        pos[right+1] = i+1;
	        right++;
	        len++;

	        while (len > 1) {
	          double d1 = s[left].evaluate(X[i]);
	          double d2 = s[left + 1].evaluate(X[i]);

	          if (d1 < d2 || Math.abs(d1-d2) < EPSILON) {
	            len--;
	            left++;
	          } else {
	            break;
	          }
	        }

	        best[i] = pos[left];      }

	      return best;
	    }
	 

	  private static double[] CalcMaxFast(Function[] f, int N, double[] X) {
	      double[] best = new double[N];
	      Function[] s = new Function[N];

	      int len = 0;
	      int left = 0;
	      int right = -1;

	      for (int i = N - 2; i >= 0; i--) {
	        while (len > 1 ) {
	          double d1 = IntersectX(f[i+1], s[right]);
	          double d2 = IntersectX(s[right], s[right - 1]);

	          if (d1 > d2 || Math.abs(d1-d2) < EPSILON) {
	            len--;
	            right--;
	          } else {
	            break;
	          }
	        }

	        s[right+1] = f[i+1];
	        right++;
	        len++;

	        while (len > 1) {
	          double d1 = s[left].evaluate(X[i]);
	          double d2 = s[left + 1].evaluate(X[i]);

	          if (d1 < d2 || Math.abs(d1-d2) < EPSILON) {
	            len--;
	            left++;
	          } else {
	            break;
	          }
	        }

	        best[i] = s[left].evaluate(X[i]);      }

	      return best;
	    }


	   private static double  Faster(int N, int K, double X[], double Y[]) {
		  	      
	      boolean iX = true;
	      
	      for(int n=0; n< N-1; n++) {
	    	  iX = iX && X[n] < X[n+1];
	      }
	      
	     // System.out.println("Increasing X :" + iX);
	      
	      boolean iS = true;
	      
	      for(int n=0; n < N-1; n++) {
	    	  iS = iS && Y[n] > Y[n+1];
	    	  
	    	  //if (!iS) {
	    	//	  System.out.println("n "+ n + " : " + (Y[n+1]-Y[n]) );
	    	//  }
	      }
	    //  System.out.println("Decreasing Y:" + iS);
	      
	      // now it is time for the real calculation
	      Function[] fun = new Function[N];

	      for(int n=0; n<N; n++) {
	        fun[n] = new Function(-Y[n], X[n] * Y[n]);
	      }
	      
	 
	      double[] best = CalcMaxFast(fun, N, X);
	      
	      
	      double res = -1;

	      for(int k=0; k<K-2; k++) {
	        Function[] F= new Function[N];

	        for(int n=0; n < N; n ++) {
	          F[n] = new Function(-Y[n], X[n] * Y[n] + best[n]);
	        }

	        best = CalcMaxFast(F, N, X);
	      }
	   

	      for(int n=0; n<N;n++) {
	        res = Math.max(res, fun[n].evaluate(0) + best[n]);
	      }

	      return res;
	    }
	   
	   

	   private static Point[]  FasterPoint(int N, int K, double X[], double Y[]) {
		  	      
	      boolean iX = true;
	      
	      
	      for(int n=0; n< N-1; n++) {
	    	  iX = iX && X[n] < X[n+1];
	      }
	      
	     // System.out.println("Increasing X :" + iX);
	      
	      boolean iS = true;
	      
	      for(int n=0; n < N-1; n++) {
	    	  iS = iS && Y[n] > Y[n+1];
	    	  
	    	  //if (!iS) {
	    	//	  System.out.println("n "+ n + " : " + (Y[n+1]-Y[n]) );
	    	//  }
	      }
	    //  System.out.println("Decreasing Y:" + iS);
	      
	      // now it is time for the real calculation
	      Function[] fun = new Function[N];

	      for(int n=0; n<N; n++) {
	        fun[n] = new Function(-Y[n], X[n] * Y[n]);
	      }
	      
	      
	      int[][] chosen = new int[K][N];
	 
	      double[] best = CalcMaxFast(fun, N, X);
	      chosen[0] = CalcMaxPoint(fun, N, X);
	      
	      double res = -1;

	      for(int k=0; k<K-2; k++) {
	        Function[] F= new Function[N];

	        for(int n=0; n < N; n ++) {
	          F[n] = new Function(-Y[n], X[n] * Y[n] + best[n]);
	        }

	        best = CalcMaxFast(F, N, X);
	        chosen[k+1] = CalcMaxPoint(F, N, X);
	      }
	    
	      
	      int startpos = 0;
	   

	      for(int n=0; n<N;n++) {
	    	   double help = fun[n].evaluate(0) + best[n];
	    	   if(help > res) {
	    		   res = help;
	    		   startpos = n;
	    	   }
	      }
	      
	      Point[] point = new Point[K];
	      
	      
	      
	      point[K-1] = new Point(X[startpos], Y[startpos]);
	      
	      for(int k=K-2; k>=0; k--) {
	    	  startpos = chosen[k][startpos];
		      point[k] = new Point(X[startpos], Y[startpos]);
	      }

	      return point;
	    }
	   
	   
	   
	   
	   public static double average(ArrayList<Double> list) {
		   double sum = 0;
		   for(int n=0; n< list.size() ; n++) {
			   sum += list.get(n);
		   }
		   
		   return sum/list.size();
	   }
	   
	   
	   

		  public static Point[] hyperCalcPoints(Point[] front,  int K, double refX, double refY ) {
			  int N = front.length;
			  // use front 100-100
			  
			  for(int n=0; n<front.length; n++) {
				  front[n].x = refX - front[n].x;
				  front[n].y = refY - front[n].y;
			  }

			  // sort by x coordinate
		      Arrays.sort(front, new Comparator<Point>(){
		          @Override
		          public int compare(Point o1, Point o2) {
		              if (o1.x < o2.x) {
		              	return -1;
		              } else  if (o1.x == o2.x){
		            	  if(o1.y < o2.y) {
		            		  return -1;
		            	  } else if(o1.y == o2.y) {
		            		  return 0;
		            	  } else {
		            		  return 1;
		            	  }
		              } else {
		            	  return 1;
		              }
		          }
		      });
		      
		     // System.out.println("EL : " + front.length + " N: "+ N);
		      
		      Stack<Integer> st = new Stack<Integer>();
		      
		      st.push(0);
		                  
		      for(int n=1; n<N; n++){
		    	  
		    	  while(st.size() > 0 && front[st.peek()].y <= front[n].y) {
		    		  st.pop();
		    	  } 
		    	  st.push(n);
		      }
		      
		      double X[] = new double[st.size()];
		      double Y[] = new double[st.size()];
		      
		      Function[] f = new Function[st.size()];
		      
		      for (int i=0; i < st.size(); i++) {
		        	int n = st.get(i);
			        double x= front[n].x;
			        double y= front[n].y;
			        X[i] = x;
			        Y[i] = y;
			        f[i] = new Function(-y, x*y);
		      }
		      		      	      
		      int size = st.size();
		      
		      

		      double [][] dp = new double[size][K+1];

		      for(int n=0; n< size; n++) {
		        for(int k=0; k < K +1; k++) {
		          dp[n][k] = -1;
		        }
		      }
		      
		      Point[] res2 = FasterPoint(size, Math.min(K, size), X, Y);	
		      
		      // reconvert points
		      for(int n=0; n<res2.length; n++) {
		    	  res2[n].x = refX - res2[n].x;
		    	  res2[n].y = refY - res2[n].y;
		      }
		      
		      // reconvert points
		      for(int n=0; n<front.length; n++) {
				  front[n].x = refX - front[n].x;
				  front[n].y = refY - front[n].y;
			  }
		      
		      // reverse order
		      for(int n=front.length-1;n>=0; n--) {
		    	  int m = front.length-1-n;
		    	  if (n < m ){
		    		  double x1 = front[n].x;
		    		  double y1 = front[n].y;
		    		  front[n].x = front[m].x;
		    		  front[n].y = front[m].y;
		    		  front[m].x = x1;
		    		  front[m].y = y1;
		    		 }
		    	  }
		    	  
		     
		      
		      
		      return res2;
		    }

	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
		  public static double hyperCalc(SolutionSet sol,  int K, double refX, double refY ) {

			  double [][] front = sol.writeObjectivesToMatrix();
			  
			  int N = front.length;
			  // use front 100-100
			  
			  for(int n=0; n<front.length; n++) {
				  front[n][0] = refX - front[n][0];
				  front[n][1] = refY - front[n][1];
			  }

			  // sort by x coordinate
		      Arrays.sort(front, new Comparator<double[]>(){
		          @Override
		          public int compare(double[] o1, double[] o2) {
		              if (o1[0] < o2[0]) {
		              	return -1;
		              } else  if (o1[0] == o2[0]){
		            	  if(o1[1] < o2[1]) {
		            		  return -1;
		            	  } else if(o1[1] == o2[1]) {
		            		  return 0;
		            	  } else {
		            		  return 1;
		            	  }
		              } else {
		            	  return 1;
		              }
		          }
		      });
		      
		     // System.out.println("EL : " + front.length + " N: "+ N);
		      
		      Stack<Integer> st = new Stack<Integer>();
		      
		      st.push(0);
		                  
		      for(int n=1; n<N; n++){
		    	  
		    	  while(st.size() > 0 && front[st.peek()][1] <= front[n][1]) {
		    		  st.pop();
		    	  } 
		    	  st.push(n);
		      }
		      
		      double X[] = new double[st.size()];
		      double Y[] = new double[st.size()];
		      
		      Function[] f = new Function[st.size()];
		      
		      for (int i=0; i < st.size(); i++) {
		        	int n = st.get(i);
			        double x= front[n][0];
			        double y= front[n][1];
			        X[i] = x;
			        Y[i] = y;
			        f[i] = new Function(-y, x*y);
		      }
		      		      	      
		      int size = st.size();

		      double [][] dp = new double[size][K+1];

		      for(int n=0; n< size; n++) {
		        for(int k=0; k < K +1; k++) {
		          dp[n][k] = -1;
		        }
		      }
		      
		      
		    //  System.out.println("N : " + size + " K : " + K);
		    //  double res = Rec(0, size, Math.min(size, K), dp, f, X);
		      double res2 = Faster(size, Math.min(K, size), X, Y);

		      //System.out.println("DIFF " + (res - res2));
		      
		      return res2;
		    }

		  
      public static double hyperCalc3(ArrayList<Point> front,  int K, double refX, double refY) {
    	  
		  int N = front.size();
		  // use front 100-100
		  
		  for(int n=0; n<N; n++) {
			  front.get(n).x = refX - front.get(n).x;
			  front.get(n).y = refY - front.get(n).y;
		  }

		  // sort by x coordinate
		  
	      Collections.sort(front, new Comparator<Point>(){
	          @Override
	          public int compare(Point o1, Point o2) {
	              if (o1.x < o2.x) {
	              	return -1;
	              } else  if (o1.x == o2.x){
	            	  if(o1.y < o2.y) {
	            		  return -1;
	            	  } else if(o1.y == o2.y) {
	            		  return 0;
	            	  } else {
	            		  return 1;
	            	  }
	              } else {
	            	  return 1;
	              }
	          }
	      });
	      
	     // System.out.println("EL : " + front.length + " N: "+ N);
	      
	      Stack<Integer> st = new Stack<Integer>();
	      
	      st.push(0);
	                  
	      for(int n=1; n<N; n++){
	    	  
	    	  while(st.size() > 0 && front.get(st.peek()).y <= front.get(n).y) {
	    		  st.pop();
	    	  } 
	    	  st.push(n);
	      }
	      
	      double X[] = new double[st.size()];
	      double Y[] = new double[st.size()];
	      
	      Function[] f = new Function[st.size()];
	      
	      for (int i=0; i < st.size(); i++) {
	        	int n = st.get(i);
		        double x= front.get(n).x;
		        double y= front.get(n).y;
		        X[i] = x;
		        Y[i] = y;
		        f[i] = new Function(-y, x*y);
	      }
	      	      
	      int size = st.size();
	      
	      

	      double [][] dp = new double[size][K+1];

	      for(int n=0; n< size; n++) {
	        for(int k=0; k < K +1; k++) {
	          dp[n][k] = -1;
	        }
	      }
	      
	      double res2 = Faster(size, Math.min(K, size), X, Y);	
	      
	      
	      // reconvert points
	      for(int n=0; n<N; n++) {
			  front.get(n).x = refX - front.get(n).x;
			  front.get(n).y = refY - front.get(n).y;
		  }
	      
	      // reverse order
	      for(int n=N-1;n>=0; n--) {
	    	  int m = N-1-n;
	    	  if (n < m ){
	    		  double x1 = front.get(n).x;
	    		  double y1 = front.get(n).y;
	    		  front.get(n).x = front.get(m).x;
	    		  front.get(n).y = front.get(m).y;
	    		  front.get(m).x = x1;
	    		  front.get(m).y = y1;
	    		 }
	    	  }	      
	      
	      return res2;
      }
	

	  
	  public static double hyperCalc2(Point[] front,  int K, double refX, double refY ) {

		  
		  int N = front.length;
		  // use front 100-100
		  
		  for(int n=0; n<front.length; n++) {
			  front[n].x = refX - front[n].x;
			  front[n].y = refY - front[n].y;
		  }

		  // sort by x coordinate
	      Arrays.sort(front, new Comparator<Point>(){
	          @Override
	          public int compare(Point o1, Point o2) {
	              if (o1.x < o2.x) {
	              	return -1;
	              } else  if (o1.x == o2.x){
	            	  if(o1.y < o2.y) {
	            		  return -1;
	            	  } else if(o1.y == o2.y) {
	            		  return 0;
	            	  } else {
	            		  return 1;
	            	  }
	              } else {
	            	  return 1;
	              }
	          }
	      });
	      
	     // System.out.println("EL : " + front.length + " N: "+ N);
	      
	      Stack<Integer> st = new Stack<Integer>();
	      
	      st.push(0);
	                  
	      for(int n=1; n<N; n++){
	    	  
	    	  while(st.size() > 0 && front[st.peek()].y <= front[n].y) {
	    		  st.pop();
	    	  } 
	    	  st.push(n);
	      }
	      
	      double X[] = new double[st.size()];
	      double Y[] = new double[st.size()];
	      
	      Function[] f = new Function[st.size()];
	      
	      for (int i=0; i < st.size(); i++) {
	        	int n = st.get(i);
		        double x= front[n].x;
		        double y= front[n].y;
		        X[i] = x;
		        Y[i] = y;
		        f[i] = new Function(-y, x*y);
	      }
	      
	    //  System.out.println("y :" + Y[st.size()-1]);
	      	      
	      int size = st.size();
	      
	      

	      double [][] dp = new double[size][K+1];

	      for(int n=0; n< size; n++) {
	        for(int k=0; k < K +1; k++) {
	          dp[n][k] = -1;
	        }
	      }
	      
	      double res2 = Faster(size, Math.min(K, size), X, Y);	
	      
	      
	      // reconvert points
	      for(int n=0; n<front.length; n++) {
			  front[n].x = refX - front[n].x;
			  front[n].y = refY - front[n].y;
		  }
	      
	      // reverse order
	      for(int n=front.length-1;n>=0; n--) {
	    	  int m = front.length-1-n;
	    	  if (n < m ){
	    		  double x1 = front[n].x;
	    		  double y1 = front[n].y;
	    		  front[n].x = front[m].x;
	    		  front[n].y = front[m].y;
	    		  front[m].x = x1;
	    		  front[m].y = y1;
	    		 }
	    	  }
	    	  
	     
	      
	      
	      return res2;
	    }
	  
	   private static double Rec(int pos, int N, int K, double[][] dp, Function f[], double[] X) {
	     //System.out.println("pos " + pos + " N : " + N);
	      if(K==0) {
	        return 0;
	      } else if (pos >= N) {
	        // TODO is this sufficient?
	        return -10000;
	      } else {
	        if(dp[pos][K] == -1) {
	          double val = 0;

	          for(int n=pos; n < N; n++){
	            double plus = f[n].evaluate(pos==0? 0 :  X[pos-1]);
	            val = Math.max(
	              val, 
	              plus + Rec(n+1, N,  K-1, dp, f, X)
	            );
	          }
	          dp[pos][K] = val;
	        }

	        return dp[pos][K];
	      }
	    }
	    
}
