package HyperEps;
import static org.junit.Assert.*;

import java.awt.List;
import java.util.ArrayList;
import java.util.Random;

import org.junit.Test;


public class AlgoTests {

	public void testEpsSimple(Point[] P1, int k) {
		Eps eps = new Eps(P1, P1);
		
		double d1 =eps.simpleSolution(true, k);
		double d2 =eps.simpleSolution(false, k);
		double d3 = eps.fastSolution(k);
		
		assertEquals(d1, d2, 1.0e-7);
		assertEquals(d2, d3, 1.0e-7);
	}
	
	public void testEps(Point[] P1, int k) {
		Eps eps = new Eps(P1, P1);
		
		double d1 =eps.simpleSolution(true, k);
		double d2 = eps.fastSolution(k);
		
		assertEquals(d1, d2, 1.0e-7);
	}
	
	@Test
	public void nonDominantTest(){
		Point p[] = {
		new Point(1,1),
		new Point(2,2),
		new Point(3, -1),
		new Point(5,1)
		};
		
		Point[] res = Eps.makeNonDominant(p);
		
		for(Point a : res) {
			System.out.println(a.x + " " + a.y);
		}
	}
	
	@Test
	public void nonDominantRandomTest(){
		
		Random rnd = new Random();
		Point p[] = new Point[1000];
		
		for(int n=0; n<1000; n++){
			p[n] = new Point(10000*rnd.nextDouble(), 10000*rnd.nextDouble());	
		}
		
		Point[] res = Eps.makeNonDominant(p);
		
		for(Point a : res) {
			System.out.println(a.x + " " + a.y);
		}
	}
	
	
	@Test
	public void simpleTests() {
		
		Point[] P1 = {new Point(1,2), new Point(2,1)};
		testEpsSimple(P1,1);
		
		for(int k=1;k<100; k++) {
			Point[] P2 = new Point[100];
			
			for(int n=0; n<100; n++){
				P2[n] = new Point(0.02 * n,  1000-3*n);
			}
			
			testEpsSimple(P2, k);
		}
	}
	
	@Test
	public void complicatedTest() {
		Point[] P1 = {new Point(1,2), new Point(2,1)};
		testEpsSimple(P1,1);
		
		for(int k=1;k<10; k++) {
			Point[] P2 = new Point[1000];
			
			for(int n=0; n<1000; n++){
				P2[n] = new Point(0.02 * n,  10000-0.1*n);
			}
			
			testEps(P2, k);
			System.out.println("K :" + k);
		}
		

	}
	
	
	

}
