package HyperEps;

import java.util.HashMap;
import java.util.Properties;

import ec.wf.algorithm.TurbineNSGAII;
import ec.wf.representation.ContinuousTurbineProblem_Power_Area_EMST;
import ec.wf.variation.BlockSwapCrossover;
import ec.wf.variation.MOTDA;
import ec.wf.variation.MovementMutation;
import ec.wf.variation.Variation;
import jmetal.core.Algorithm;
import jmetal.core.Operator;
import jmetal.core.Problem;
import jmetal.core.SolutionSet;
import jmetal.metaheuristics.ibea.IBEA;
import jmetal.metaheuristics.ibea.TurbineIBEA;
import jmetal.metaheuristics.moead.MOEAD;
import jmetal.metaheuristics.moead.TurbineMOEAD;
import jmetal.metaheuristics.nsgaII.NSGAII;
import jmetal.metaheuristics.smsemoa.SMSEMOA;
import jmetal.metaheuristics.smsemoa.TurbineSMSEMOA;
import jmetal.metaheuristics.spea2.SPEA2;
import jmetal.metaheuristics.spea2.TurbineSPEA2;
import jmetal.operators.crossover.CrossoverFactory;
import jmetal.operators.mutation.MutationFactory;
import jmetal.operators.selection.BinaryTournament;
import jmetal.operators.selection.SelectionFactory;
import jmetal.util.JMException;
import jmetal.util.comparators.FitnessComparator;

public class AlgorithmManager extends Algorithm {

	Algorithm algorithm;
	Problem problem;
	
	public int selectionSize;//100;

	
	public AlgorithmManager(Problem p, Problem subProblem, Algorithm alg, 
			Properties config) throws ClassNotFoundException, JMException{
		super(p);
		
		int maxEval =  Integer.parseInt(config.getProperty("MaxEvaluations"));
		int selectionSize = Integer.parseInt(config.getProperty("PopulationSize"));
		
		problem = p;
		algorithm = alg;
		this.selectionSize = selectionSize;
		
	    HashMap parameters;
	    Operator crossover, mutation, selection;
	    
		  algorithm.setInputParameter("populationSize",selectionSize);
		  algorithm.setInputParameter("maxEvaluations",maxEval);
		   
		if (subProblem instanceof TTPProblem) {
			System.out.println("TTPProblem");
			
			if (algorithm instanceof SMSEMOA || algorithm instanceof NSGAII) {
			    algorithm.setInputParameter("offset", 10.0);

				parameters = new HashMap();
				parameters.put("probability", 0.1) ;
				parameters.put("distributionIndex", 20.0) ;
				
				
				double prob1 = 0.6;
				double prob2 = 0.3;
				
			    crossover = new TTPCrossover(prob1, prob2, parameters);    
			    
			    // bit flipper
			    parameters = new HashMap() ;
			    double  elements = 3;
			    			    
			        double  expectedFlips = 30;
				    
				    int numCities = ((TTPProblem) subProblem).numCities;
				    
				    prob1 = 0.5;
				    prob2 = expectedFlips * (1.0/(numCities));
			    
			    
			    mutation = new TTPMutation(prob1, prob2, parameters);                    

			    // Selection Operator 
			    parameters = null ;
			    selection = SelectionFactory.getSelectionOperator("BinaryTournament2", parameters) ;  

			    algorithm.addOperator("crossover", crossover);
			    algorithm.addOperator("mutation", mutation);
			    algorithm.addOperator("selection", selection);
			} else {
				throw new ClassNotFoundException("AlgorithmManager is not implemented for this class");
			}
			
		} else if (subProblem instanceof ContinuousTurbineProblem_Power_Area_EMST) {
			
			if (algorithm instanceof TurbineNSGAII ||
				algorithm instanceof TurbineIBEA ||
				algorithm instanceof TurbineSPEA2 ||
				algorithm instanceof TurbineMOEAD ||
				algorithm instanceof TurbineSMSEMOA) {
				
				parameters = new HashMap();
				
				String algorithmStr = config.getProperty("Algorithm");
				config.getProperty("SubAlgorithm");
				int arcSize = Integer.parseInt(config.getProperty("ArchiveSize"));
				int popSize = Integer.parseInt(config.getProperty("PopulationSize"));
				int maxEvals = Integer.parseInt(config.getProperty("MaxEvaluations"));
				// Problem
				int numberOfTurbines = Integer.parseInt(config.getProperty("NumberOfTurbines"));
				int wakeModelScenario = Integer.parseInt(config.getProperty("WakeModelScenario"));

				// Objectives
				String objectiveStr = config.getProperty("ObjectiveType");
				if(objectiveStr == null)
					objectiveStr = "";
				
				// Operators
				String mutationTypeStr = config.getProperty("MutationType");
				Double mutationProb = Double.parseDouble(config.getProperty("MutationProb"));
				String crossoverTypeStr = config.getProperty("CrossoverType");
				Double crossoverProb = Double.parseDouble(config.getProperty("CrossoverProb"));
				
				
			    parameters.put("probability", crossoverProb);
			    parameters.put("type", "CONT");									//since it will always be continuous
				
			    if (crossoverTypeStr.equals(Variation.crossoverTypes[0])) {
					crossover = new BlockSwapCrossover(parameters);
				} else {
					crossover = new BlockSwapCrossover(parameters);
				}
				
				parameters = new HashMap();
			    parameters.put("probability", mutationProb);
			    parameters.put("type", "CONT");									//since it will always be continuous
				if (mutationTypeStr.equals(Variation.mutationTypes[0])) {
					mutation = new MOTDA(parameters);
				} else {
					mutation = new MovementMutation(parameters);
				}

			    // Selection Operator
			    parameters = null;
			    try {
					selection = SelectionFactory.getSelectionOperator("BinaryTournament2", parameters);
				
					algorithm.setInputParameter("populationSize",popSize);
					algorithm.setInputParameter("maxEvaluations",maxEvals);
				    algorithm.setInputParameter("archiveSize",arcSize);
				    
				    // Add the operators to the algorithm
				    algorithm.addOperator("crossover",crossover);
				    algorithm.addOperator("mutation",mutation);
				    algorithm.addOperator("selection",selection);
				} catch (JMException e) {
					e.printStackTrace();
				} 
			} else {
				throw new ClassNotFoundException("AlgorithmManager is not implemented for this class");
			}
			
			
			
		}
		else {
		    
		    
		if (algorithm instanceof NSGAII) {
			
			    // Mutation and Crossover for Real codification 
			    parameters = new HashMap() ;
			    parameters.put("probability", 0.9) ;
			    parameters.put("distributionIndex", 20.0) ;
			    crossover = CrossoverFactory.getCrossoverOperator("SBXCrossover", parameters);                   

			    parameters = new HashMap() ;
			    parameters.put("probability", 1.0/problem.getNumberOfVariables()) ;
			    parameters.put("distributionIndex", 20.0) ;
			    mutation = MutationFactory.getMutationOperator("PolynomialMutation", parameters);                    

			    // Selection Operator 
			    parameters = null ;
			    selection = SelectionFactory.getSelectionOperator("BinaryTournament2", parameters) ;                           
			
		} else if (algorithm instanceof MOEAD) {
		
			    
			    // Directory with the files containing the weight vectors used in 
			    // Q. Zhang,  W. Liu,  and H Li, The Performance of a New Version of MOEA/D 
			    // on CEC09 Unconstrained MOP Test Instances Working Report CES-491, School 
			    // of CS & EE, University of Essex, 02/2009.
			    // http://dces.essex.ac.uk/staff/qzhang/MOEAcompetition/CEC09final/code/ZhangMOEADcode/moead0305.rar
			    algorithm.setInputParameter("dataDirectory",
			    "/Users/antelverde/Softw/pruebas/data/MOEAD_parameters/Weight");

			    algorithm.setInputParameter("finalSize", 300) ; // used by MOEAD_DRA

			    algorithm.setInputParameter("T", 20) ;
			    algorithm.setInputParameter("delta", 0.9) ;
			    algorithm.setInputParameter("nr", 2) ;

			    // Crossover operator 
			    parameters = new HashMap() ;
			    parameters.put("CR", 1.0) ;
			    parameters.put("F", 0.5) ;
			    crossover = CrossoverFactory.getCrossoverOperator("DifferentialEvolutionCrossover", parameters);                   
			    
			    // Mutation operator
			    parameters = new HashMap() ;
			    parameters.put("probability", 1.0/problem.getNumberOfVariables()) ;
			    parameters.put("distributionIndex", 20.0) ;
			    mutation = MutationFactory.getMutationOperator("PolynomialMutation", parameters);                    
			    
			    algorithm.addOperator("crossover",crossover);
			    algorithm.addOperator("mutation",mutation);
			    
			    // no selection, but should not do anything
			    parameters = null ;
			    selection = SelectionFactory.getSelectionOperator("BinaryTournament2", parameters) ;                           
			
			    	
		}
		else if (algorithm instanceof SMSEMOA) {
			
			// Algorithm parameters
		    algorithm.setInputParameter("offset", 10.0);

		    // Mutation and Crossover for Real codification 
		    parameters = new HashMap() ;
		    parameters.put("probability", 0.9) ;
		    parameters.put("distributionIndex", 20.0) ;
		    crossover = CrossoverFactory.getCrossoverOperator("SBXCrossover", parameters);

		    parameters = new HashMap() ;
		    parameters.put("probability", 1.0/problem.getNumberOfVariables()) ;
		    parameters.put("distributionIndex", 20.0) ;
		    mutation = MutationFactory.getMutationOperator("PolynomialMutation", parameters);

		    // Selection Operator
		    parameters = null ;
		    selection = SelectionFactory.getSelectionOperator("RandomSelection", parameters);
		    // also possible
		    //selection = SelectionFactory.getSelectionOperator("BinaryTournament2");

		    // Add the operators to the algorithm
		  
		}
		else if (algorithm instanceof IBEA) {
		    algorithm.setInputParameter("archiveSize",selectionSize);

		    // Mutation and Crossover for Real codification 
		    parameters = new HashMap() ;
		    parameters.put("probability", 0.9) ;
		    parameters.put("distributionIndex", 20.0) ;
		    crossover = CrossoverFactory.getCrossoverOperator("SBXCrossover", parameters);                   

		    parameters = new HashMap() ;
		    parameters.put("probability", 1.0/problem.getNumberOfVariables()) ;
		    parameters.put("distributionIndex", 20.0) ;
		    mutation = MutationFactory.getMutationOperator("PolynomialMutation", parameters);         

		    /* Selection Operator */
		    parameters = new HashMap() ; 
		    parameters.put("comparator", new FitnessComparator()) ;
		    selection = new BinaryTournament(parameters);		
		} else if (algorithm instanceof SPEA2) {
		    
		    // Algorithm parameters
		    algorithm.setInputParameter("archiveSize",selectionSize);
		      
		    // Mutation and Crossover for Real codification 
		    parameters = new HashMap() ;
		    parameters.put("probability", 0.9) ;
		    parameters.put("distributionIndex", 20.0) ;
		    crossover = CrossoverFactory.getCrossoverOperator("SBXCrossover", parameters);                   

		    parameters = new HashMap() ;
		    parameters.put("probability", 1.0/problem.getNumberOfVariables()) ;
		    parameters.put("distributionIndex", 20.0) ;
		    mutation = MutationFactory.getMutationOperator("PolynomialMutation", parameters);                    
		        
		    // Selection operator 
		    parameters = null ;
		    selection = SelectionFactory.getSelectionOperator("BinaryTournament", parameters) ;                            
		}
		else {
			throw new ClassNotFoundException("AlgorithmManager is not implemented for this class");
		}
		
		  algorithm.addOperator("crossover", crossover);
		    algorithm.addOperator("mutation", mutation);
		    algorithm.addOperator("selection", selection);
		}
	}
	
	@Override
	public SolutionSet execute() throws JMException, ClassNotFoundException {
		return algorithm.execute();
	}

}
