package HyperEps;

import java.security.InvalidAlgorithmParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.Stack;

import jmetal.core.SolutionSet;


public class Eps {
	
	
	public static Point[] makeNonDominant(Point[] front){
			int N = front.length;
			
		     Arrays.sort(front, new Comparator<Point>(){
		          @Override
		          public int compare(Point o1, Point o2) {
		              if (o1.x < o2.x) {
		              	return -1;
		              } else  if (o1.x == o2.x){
		            	  if(o1.y < o2.y) {
		            		  return -1;
		            	  } else if(o1.y == o2.y) {
		            		  return 0;
		            	  } else {
		            		  return 1;
		            	  }
		              } else {
		            	  return 1;
		              }
		          }
		      });
		      
		      Stack<Integer> st = new Stack<Integer>();
		      
		      st.push(N-1);
		                  
		      for(int n=N-2; n>=0; n--){
		    	  while(st.size() > 0 && front[st.peek()].y >= front[n].y) {
		    		  st.pop();
		    	  } 
		    	  st.push(n);
		      }
		      
		      Point p[] = new Point[st.size()];
		      
		      for (int i=0; i < st.size(); i++) {
		        	int n = st.get(i);
		        	//System.out.println("st :" +n);
			        p[st.size()-1-i] = front[n];
		      }
		      
		 //    System.out.println("x : " + p[st.size()-1].x +" " + p[st.size()-1].y);
		     return p;
		}
	
	   
		   
	
	
	
	
	public static double I(Point p1, Point p2) {
		return Math.max(
			p2.x - p1.x,
			p2.y - p1.y
		);
	}
	/**
	 * Take elements of B to approximate A
	 * @param P1
	 * @param P2
	 */
	public Eps(Point[] A, Point[] B) {
		this.A = A;
		this.B = B;
		rand = new Random();
	}
	
	Point[] A;
	Point[] B;
    Random rand;


 	
   public boolean epsilonTest(double epsilon, int k) {
	   boolean res = true;
	   try{
		   this.epsilonPoints(epsilon, k);
	   }catch (InvalidAlgorithmParameterException ex)  {
		   res = false;
	   }

	   return res;
    }
   
   
   // we want k points of B to dominate A , is  epsilon enough
   public Point[] epsilonPoints(double epsilon, int k) throws InvalidAlgorithmParameterException {
	   int posA = 0;
	   int posB = 0;
	   
	   ArrayList<Point> p = new ArrayList<Point>();

	      while(posB < B.length && posA < A.length) {
	        k--;
	        while (posB +1 < B.length && B[posB+1].x - A[posA].x <= epsilon  ) {
	        	posB++;
	        }
	        	        
	        // delete all A points dominates by the point
	        int help = posA;
	        while (help < A.length && I(A[help], B[posB]) <= epsilon) {
	          help++;
	        }
	        
	        p.add(B[posB]);

	        if (help == posA) {
	          // no approximation :(
	        	throw new InvalidAlgorithmParameterException();
	        }
	        posA = help;
	      }
	      	      
	      // return the sorted Points
	      if (posA == A.length) {
	    	 if(k >= 0) {
	    		 Point[] points = new Point[p.size()];
	    		 for(int n=0; n<p.size();n++) {
	    			 points[n] = p.get(n);
	    		 }
	    		 return points;
	    	 }
	      }
	      
	       throw new InvalidAlgorithmParameterException();
    }

	
	private List<Set> createSet() {

	      List<Set> v = new ArrayList<Set>();

	      int a = -1;
	      for(Point pointB : B ) {
	        double m = 1;
	        double n = pointB.y - pointB.x;
	        // f(x) 
	        
	        // A.y >= A.x
	        
	        // TODO : what is with an empty size
	        while(a+1 < A.length && (m*A[a+1].x + n) <= A[a+1].y) {
	          a++;          
	        }
	        
	        
	        
	        v.add(new Set(pointB, A, 0, a, true));
	        v.add(new Set(pointB, A, a+1, A.length-1, false));
	      }

	      return v;
	    }
	 
	 public long randLong(long max) {
		 long range = max;
		 long number = (long)(rand.nextDouble()*range);
		 return number;
	 }
	 
	 public double fastSolution(int k) {
		    List<Set> v = createSet();

		    // TODO : change this to a resonable value
		    double best = 10000000;

		      int it =0;
		      while(true) {
		        it++;

		        long totalSize = 0;
		        for (Set s : v) {
		          totalSize += s.getSize();
		        }

		
		        if (totalSize == 0) {
		          return best;
		        }


		        long num = randLong(totalSize);
		        
		        // System.out.println("num " + num + " " + totalSize + " " + v.size());
		        
		        if (it > 100 && totalSize < 100) {
		        	// this is a hack
		        	return best;
		        }
		 

		        // choose element num from the set
		        double val = 0;
		        for(Set s : v) {
		          if (s.getSize() <= num) {
		            num -= s.getSize();
		          } else {
		            val = s.getElement((int)num);
		            break;
		          }
		        }
		        if (epsilonTest(val, k)) {
		
		          if (val < best) {
		            best = val;
		          } 

		          for (int n=0; n<v.size(); n++) {
		            v.get(n).deleteBigger(best);
		          }



		          // e >= e*
		        } else {
		
		         for (int n=0; n<v.size(); n++) {
		            v.get(n).deleteSmaller(val);
		          }
		        }
		        
		        // delete empty Set
		        ArrayList<Set> newList = new ArrayList<>();
		        for(Set s : v) {
		        	if (s.getSize() > 0) {
		        		newList.add(s);
		        	}
		        }
		        
		        v = newList;
		      }
		    }

		    public double simpleSolution(boolean binarySearch, int k) {

		      ArrayList<Double> possibleValues = new ArrayList<Double>();

		      for (Point a : A) {
		        for( Point b : B) {
		          possibleValues.add(I(a,b));
		        }
		      }
		      
		      Collections.sort(possibleValues);
		      
		      int begin = 0;
		      int end = possibleValues.size() - 1;

		      if (binarySearch) {
		        while(begin < end) {
		          int mid = (begin + end) / 2;
		          if (epsilonTest(possibleValues.get(mid), k)) {
		            end = mid;
		          } else {
		            begin = mid + 1;
		          }
		        }

		        // cout<<"begin "<<begin<<endl;

		        return possibleValues.get(begin);
		      } else {

		        for (double dist : possibleValues) {
		          if (epsilonTest(dist, k)) {
		            return dist;
		          }
		        }
		      }

		      return -1;
		    }	
};
