package HyperEps;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Properties;

import jmetal.core.Algorithm;
import jmetal.core.Problem;
import jmetal.metaheuristics.ibea.IBEA;
import jmetal.metaheuristics.moead.MOEAD;
import jmetal.metaheuristics.nsgaII.NSGAII;
import jmetal.metaheuristics.smsemoa.SMSEMOA;
import jmetal.metaheuristics.spea2.SPEA2;
import jmetal.problems.DTLZ.DTLZ1;
import jmetal.problems.DTLZ.DTLZ2;
import jmetal.problems.DTLZ.DTLZ7;
import jmetal.problems.ZDT.ZDT1;
import jmetal.problems.ZDT.ZDT2;
import jmetal.problems.ZDT.ZDT3;
import jmetal.problems.ZDT.ZDT6;
import jmetal.util.JMException;

public class MyBestOf {

	public static void main(String[] args) throws ClassNotFoundException, JMException, FileNotFoundException {
	      
		
		String current = System.getProperty("user.home") + "/hypselection/experiments/";	
	    String configFile = current + "configBestOf/" +args[0];
		
		Properties config = new Properties();
		try {
			config.load(new FileInputStream(configFile));
		} catch (IOException e) {
			e.printStackTrace();
			System.err.println("Failed to load configuration file!");
			System.exit(-1);
		}
		
		   int maxEvaluations = Integer.parseInt(config.getProperty("MaxEvaluations"));
		      int selectionSize = Integer.parseInt(config.getProperty("PopulationSize"));
		    
		      
		      
		      ArrayList<Point[]> list = new ArrayList<>();

		
		double optHyp = 0;
		
		Problem problem;
		
		switch(config.getProperty("problem")) {
			case "dtlz1":	problem = new DTLZ1("Real", 7, 2); break;
			case "dtlz2":	problem = new DTLZ2("Real", 7, 2); break;
			case "dtlz7":	problem = new DTLZ7("Real", 7, 2); break;
			case "zdt1" : problem = new ZDT1("Real"); break;
			case "zdt2" : problem = new ZDT2("Real"); break;
			case "zdt3" : problem = new ZDT3("Real"); break;
			case "zdt6" : problem = new ZDT6("Real"); break;
			default : throw new IllegalArgumentException("error in config file, no known problem");
		}
		
	      Algorithm algo[] = {
	    		  new NSGAII(problem), 
	    		  new SMSEMOA(problem), 
	    		  new IBEA(problem),
	    		  new SPEA2(problem),
	    		  new MOEAD(problem),
	      };
	      
	      for(Algorithm alg : algo ) {
	    	  
	      String[] s = alg.getClass().toString().split("\\.");
	      String name = s[s.length-1];

	      ProblemManager myproblem = new ProblemManager(problem, config);
	      
	      Algorithm algorithm;
	    
	      if (alg instanceof SMSEMOA) {
	    	  algorithm = new SMSEMOA(myproblem);
	      } else if (alg instanceof NSGAII) {
		      algorithm = new NSGAII(myproblem);
	      } else if (alg instanceof IBEA) {
	    	  algorithm = new IBEA(myproblem);
	      } else if (alg instanceof SPEA2) {
	    	  algorithm = new SPEA2(myproblem);
	      } else if (alg instanceof MOEAD) {
	    	  algorithm = new MOEAD(myproblem);
	      }
	      else {
	    	  throw new ClassNotFoundException();
	      }
	      

						
	      AlgorithmManager myAlgorithm =  new AlgorithmManager(myproblem, problem, algorithm, config);
	      
	      System.out.println("Start the algorithm");
	      long initTime = System.currentTimeMillis();
		  algorithm.execute();
		  long estimatedTime = System.currentTimeMillis() - initTime;
		  
		  Point[] rawPoints = myproblem.getRawPointSet();
		  
		  list.add(rawPoints);
		  System.out.println("Point Size :" + rawPoints.length);
		  
		  myproblem.closeAndWriteGNUPLOT();
		  
		  optHyp = myproblem.getOptHyp();
		  
		  System.out.println("Algorithm finished");
	      }
	      
	      System.out.println("Everything is done");
	      
	  	String printPath = System.getProperty("user.home") + "/bestof/bestof-" + config.getProperty("problem");
		PrintWriter writer = new PrintWriter(new File(printPath));
	      System.out.println("Write " + writer);
	    	  for (int k=4000; k<list.get(0).length; k+=100) {
	    		  //System.out.print("k : "  + k);
	    		  writer.print(k);
	    	      for(int n=0; n<list.size(); n++) {

	    		  Point[] help = new Point[k];
	    		  
	    		  for(int j=0; j<k;j++){
	    			  help[j] = list.get(n)[j];
	    		  }
	    		  
		    	  
		    	  double after = optHyp - Algo.hyperCalc2(Eps.makeNonDominant(help), selectionSize, 11, 11);
		    	 System.out.print(" " +after);
		    	  writer.print(" " + after);
	    	  }
	    	      
	    	  // best of all
	    	  Point[] set = new Point[k* list.size()];
	    	  
	    	  int counter = 0;
	    	  
	    	  for(int i=0; i< list.size() ; i++) {
	    		  for(int j=0; j<k;j++){
	    			  set[counter++] = list.get(i)[j];
	    		  }
	    	  }
	    	  
	    	  double best = optHyp - Algo.hyperCalc2(Eps.makeNonDominant(set), selectionSize, 11, 11);
	    	  System.out.print(" " + best);
	    	  
	    	  writer.print(" " + best);
	    	  System.out.println();
	    	  writer.println();
	      }
	    writer.close();

	}
	      

}
