package HyperEps;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import ec.wf.algorithm.TurbineNSGAII;
import jmetal.core.Algorithm;
import jmetal.core.Problem;
import jmetal.metaheuristics.ibea.IBEA;
import jmetal.metaheuristics.ibea.TurbineIBEA;
import jmetal.metaheuristics.moead.MOEAD;
import jmetal.metaheuristics.moead.TurbineMOEAD;
import jmetal.metaheuristics.nsgaII.NSGAII;
import jmetal.metaheuristics.smsemoa.SMSEMOA;
import jmetal.metaheuristics.smsemoa.TurbineSMSEMOA;
import jmetal.metaheuristics.spea2.SPEA2;
import jmetal.metaheuristics.spea2.TurbineSPEA2;
import jmetal.problems.DTLZ.DTLZ1;
import jmetal.problems.DTLZ.DTLZ2;
import jmetal.problems.DTLZ.DTLZ7;
import jmetal.problems.ZDT.ZDT1;
import jmetal.problems.ZDT.ZDT2;
import jmetal.problems.ZDT.ZDT3;
import jmetal.problems.ZDT.ZDT6;
import jmetal.problems.WFG.WFG3;
import jmetal.problems.WFG.WFG4;
import jmetal.problems.WFG.WFG5;
import jmetal.problems.WFG.WFG6;
import jmetal.problems.WFG.WFG7;

public class MyConfiguration extends Properties {
	public MyConfiguration(String path) {
		super();
		String current = System.getProperty("user.home") + "/hypselection/experiments/";
		current += path;
		try {
			super.load(new FileInputStream(current));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public Problem getProblem() {
		try{
		switch(this.getProperty("problem")) {
		case "dtlz1":	return new DTLZ1("Real", 7, 2); 
		case "dtlz2":	return new DTLZ2("Real", 7, 2); 
		case "dtlz7":	return new DTLZ7("Real", 7, 2); 
		case "zdt1" : return new ZDT1("Real"); 
		case "zdt2" : return new ZDT2("Real"); 
		case "zdt3" : return new ZDT3("Real"); 
		case "zdt6" : return new ZDT6("Real"); 
		case "wfg3" : return new WFG3("Real"); 
		case "wfg4" : return new WFG4("Real"); 
		case "wfg5" : return new WFG5("Real"); 
		case "wfg6" : return new WFG6("Real"); 
		case "wfg7" : return new WFG7("Real"); 
		default : throw new IllegalArgumentException("error in config file, no known property problem");
	}
		}catch(Exception ex) {
			 throw new IllegalArgumentException("error in config file, no known property problem");
		}
	}
	
	public Algorithm getAlgorithm(Problem problem) {
		
          try {
			switch(this.getProperty("algo")) {
			case "nsgaii":	return new NSGAII(problem); 
			case "ibea":	return new IBEA(problem); 
			case "spea2":	return new SPEA2(problem); 
			case "moead":	return new MOEAD(problem); 
			case "smsemoa":	return new SMSEMOA(problem); 
			case "turbine-nsgaii": return new TurbineNSGAII(problem, null);
			case "turbine-ibea": return new TurbineIBEA(problem);
			case "turbine-spea2": return new TurbineSPEA2(problem);
			case "turbine-moead": return new TurbineMOEAD(problem);
			case "turbine-smsemoa": return new TurbineSMSEMOA(problem);
			default : throw new IllegalArgumentException("error in config file, no known property algo");
		}
          } catch(Exception ex) {
        	  throw new IllegalArgumentException("error in config file, no known property algo");
          }
	}
}
