package HyperEps;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.FileHandler;

import ec.wf.algorithm.TurbineNSGAII;
import ec.wf.algorithm.ResultsRecorder;
import ec.wf.representation.ContinuousTurbineProblem_Power_Area;
import ec.wf.representation.ContinuousTurbineProblem_Power_Area_EMST;
import ec.wf.representation.ContinuousTurbineProblem_Power_EMST;
import ec.wf.representation.ContinuousTurbineProblem_Power_Perimeter;
import ec.wf.representation.TurbineArea;
import ec.wf.variation.BlockSwapCrossover;
import ec.wf.variation.MOTDA;
import ec.wf.variation.MovementMutation;
import ec.wf.variation.Variation;
import ec.wind.AbstractWakeModel;
import ec.wind.WakeModelFaster;
import jmetal.core.Algorithm;
import jmetal.core.Operator;
import jmetal.core.Problem;
import jmetal.core.SolutionSet;
import jmetal.metaheuristics.ibea.IBEA;
import jmetal.metaheuristics.ibea.TurbineIBEA;
import jmetal.metaheuristics.smsemoa.SMSEMOA;
import jmetal.metaheuristics.spea2.SPEA2;
import jmetal.operators.selection.SelectionFactory;
import jmetal.problems.DTLZ.DTLZ1;
import jmetal.problems.DTLZ.DTLZ2;
import jmetal.problems.ZDT.ZDT1;
import jmetal.problems.ZDT.ZDT2;
import jmetal.util.Configuration;
import jmetal.util.JMException;

// class for testing the turbine
public class MyTurbine {

	public static void main(String[] args) throws ClassNotFoundException,
			JMException {
		
		System.out.println("Args : " + args.length);
		
		if (args.length != 1) {
			throw new IllegalArgumentException("first argument please give test file name");
		}
				  
		 MyConfiguration config = new MyConfiguration("configTurbine/" + args[0]);

	     
			
		    Problem problem = null; 	// The problem to solve
		    Algorithm algorithm = null; // The algorithm to use
		    Operator crossover = null; 	// Crossover operator
		    Operator mutation = null; 	// Mutation operator
		    Operator selection = null; 	// Selection operator
		    
		    HashMap parameters = null; // Operator parameters
			
		    // Algorithm
			String algorithmStr = config.getProperty("Algorithm");
			config.getProperty("SubAlgorithm");
			int arcSize = Integer.parseInt(config.getProperty("ArchiveSize"));
			int popSize = Integer.parseInt(config.getProperty("PopulationSize"));
			int maxEvals = Integer.parseInt(config.getProperty("MaxEvaluations"));
			// Problem
			int numberOfTurbines = Integer.parseInt(config.getProperty("NumberOfTurbines"));
			int wakeModelScenario = Integer.parseInt(config.getProperty("WakeModelScenario"));
			
			// Objectives
			String objectiveStr = config.getProperty("ObjectiveType");
			if(objectiveStr == null)
				objectiveStr = "";
			
			// Operators
			String mutationTypeStr = config.getProperty("MutationType");
			Double mutationProb = Double.parseDouble(config.getProperty("MutationProb"));
			String crossoverTypeStr = config.getProperty("CrossoverType");
			Double crossoverProb = Double.parseDouble(config.getProperty("CrossoverProb"));
			
			// Set Up
			double areaWidth = Double.parseDouble(config.getProperty("AreaWidth"));
			double areaHeight = Double.parseDouble(config.getProperty("AreaHeight"));
			TurbineArea ta = new TurbineArea(numberOfTurbines, areaWidth, areaHeight);
			AbstractWakeModel wm = new WakeModelFaster(wakeModelScenario, "w");
			
			//make it a continuous problem

			// With Power + Perimeter objectives (Default case when
			// objectives were not specified)
			problem = new ContinuousTurbineProblem_Power_Area_EMST(
					numberOfTurbines, ta, wm);

		    // Execute the Algorithm

			String name = "pump";

			ProblemManager myproblem = new ProblemManager(problem, config);
			algorithm = config.getAlgorithm(myproblem);// new TurbineIBEA(myproblem);
			
			AlgorithmManager myAlgorithm =
					new AlgorithmManager(myproblem, problem, algorithm, config);
					
					
			System.out.println("Start the algorithm");
			long initTime = System.currentTimeMillis();
			algorithm.execute();
			long estimatedTime = System.currentTimeMillis() - initTime;

			myproblem.closeAndWriteGNUPLOT();

			System.out.println("Algorithm finished");
	}
}
