package HyperEps;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Stack;

import ec.wf.representation.ContinuousTurbineProblem;
import ec.wf.representation.ContinuousTurbineSolutionType;
import jmetal.core.Problem;
import jmetal.core.Solution;
import jmetal.core.SolutionSet;
import jmetal.encodings.solutionType.BinaryRealSolutionType;
import jmetal.encodings.solutionType.RealSolutionType;
import jmetal.problems.DTLZ.DTLZ1;
import jmetal.problems.DTLZ.DTLZ2;
import jmetal.problems.DTLZ.DTLZ7;
import jmetal.problems.WFG.WFG1;
import jmetal.problems.WFG.WFG2;
import jmetal.problems.WFG.WFG3;
import jmetal.problems.WFG.WFG4;
import jmetal.problems.WFG.WFG5;
import jmetal.problems.WFG.WFG6;
import jmetal.problems.WFG.WFG7;
import jmetal.problems.ZDT.ZDT1;
import jmetal.problems.ZDT.ZDT2;
import jmetal.problems.ZDT.ZDT3;
import jmetal.problems.ZDT.ZDT6;
import jmetal.util.JMException;

public class ProblemManager extends Problem {

  Problem problem;
  SolutionSet lastPopulation;


  ArrayList<Point> sol;

  Point[] approx;
  int selectionSize;
  int maxEvaluations;

  double optEps;
  double optHyp;
  double lowerHyp;
  double upperHyp;

  String fileName;

  static int sampleSize = 100000;
  static double epsilonLength;

  double refX = 11;
  double refY = 11;

  PrintWriter hypWriter;


  PrintWriter epsWriter;
  String hypName;
  String epsName;
  Stack<Integer> stEval;
  Properties config;

  long initTime;
  long evalTime;
  long analyzeTime;

  public interface FrontEval {
    double eval(double x);
  }

  private void CalcApprox() throws ClassNotFoundException{
    System.out.println("Start eps-approx");


    FrontEval fun;

    double[] beginFront = {0};
    double[] endFront = {1};


    if(problem instanceof DTLZ1) {
      optHyp = 120.873737	; //120 + 7.0/8.0 - 0.125 / (selectionSize-1);

      endFront = new double[] {0.5};

      fun =  new FrontEval() {
        public double eval(double x) {
          return 0.5 - x;
        }
      };
    } else if (problem instanceof DTLZ2) {
      optHyp = 120.210644;

      fun =  new FrontEval() {
        public double eval(double x) {
          return Math.sqrt(1-x*x);
        }
      };
    } else if (problem instanceof DTLZ7) {

      beginFront =new double[] {
        0,
          .6316265307,
          1.3596178367,
          2.0518383519
      };

      endFront =new double[] {
        .2514118360,
          .8594008566,
          1.5148392681,
          2.116426807
      };

      fun =  new FrontEval() {
        public double eval(double x) {
          return 4-x * (1 + Math.sin(3 * Math.PI * x));
        }
      };

    } else if (problem instanceof ZDT1) {
      optHyp = 120.662137;

      fun =  new FrontEval() {
        public double eval(double x) {
          return 1-Math.sqrt(x);
        }
      };
    } else if (problem instanceof ZDT2) {
      optHyp = 120.328881;
      ;
      fun =  new FrontEval() {
        public double eval(double x) {
          return 1-Math.pow(x,2);
        }
      };
    } else if (problem instanceof ZDT3) {

      beginFront =new double[] {
        0,
          .1822287280,
          .4093136748,
          .6183967944,
          .8233317983
      };

      endFront =new double[] {
        0.830015349e-1,
          .2577623634,
          .4538821041,
          .6525117038,
          .8518328654
      };

      fun =  new FrontEval() {
        public double eval(double x) {
          return 1-Math.sqrt(x) - x * Math.sin(10 * Math.PI * x);
        }
      };

    }
    else if (problem instanceof ZDT6) {
      beginFront =new double[] {
        .2807753191
      };

      endFront =new double[] {
        1
      };

      fun =  new FrontEval() {
        public double eval(double x) {
          return 1 - x*x;
        }
      };
    }
    else if (problem instanceof WFG3) {

      beginFront =new double[] {
        0
      };

      endFront =new double[] {
        2
      };


      fun =  new FrontEval() {
        public double eval(double x) {
          return 4 -2*x;
        }
      };
    } else if (
        problem instanceof WFG4 || 
        problem instanceof WFG5 || 
        problem instanceof WFG6 || 
        problem instanceof WFG7 ) {

      beginFront =new double[] {
        0
      };

      endFront =new double[] {
        2
      };


      fun =  new FrontEval() {
        public double eval(double x) {
          return Math.sqrt(16-4*x*x);
        }
      };
        }
    else if (problem instanceof TTPProblem){
      return;
    } else if (problem instanceof ContinuousTurbineProblem) {
      return;
    }
    else {
      throw new ClassNotFoundException("Class not found for approx");
    }

    ArrayList<Point> list = new ArrayList<>();
    epsilonLength = 0;

    for (int i=0; i<beginFront.length; i++) {
      double frontLength = endFront[i]- beginFront[i];

      int length = sampleSize/beginFront.length;
      epsilonLength = Math.max(epsilonLength, frontLength / (length-1));

      for(int n=0; n<length; n++){
        double start = beginFront[i] + (frontLength*n)/(length -1);
        double x = start;
        double y = fun.eval(x);
        list.add(new Point(x, y));
      }
    }

    approx = new Point[list.size()];

    for (int n=0; n< list.size() ; n++) {
      approx[n] = list.get(n);
    }

    System.out.println("hyperCalc2");
    lowerHyp = 	Algo.hyperCalc2(approx, selectionSize, refX, refY);

    // calculate upper
    for (int n=0; n<approx.length; n++) {
      approx[n].y = approx[n].y - epsilonLength;
    }

    upperHyp = Algo.hyperCalc2(approx, selectionSize, refX, refY);

    for (int n=0; n<approx.length; n++) {
      approx[n].y = approx[n].y + epsilonLength;
    }

    // System.out.println("Result : " + optHyp +" "+ lowerHyp + " " + upperHyp);
    System.out.println("HypDifference : " + (upperHyp- lowerHyp) );

    System.out.println("HA");
    optHyp = lowerHyp;
    
    
    System.out.println("HA");

    System.out.println("LINE : " + approx.length);
    // TODO why is this necessary ?????????????????
   // approx = Eps.makeNonDominant(approx);
    System.out.println("LINE : " + approx.length);

    Eps eps = new Eps(approx, approx);
    System.out.println("HA");
    
    optEps = eps.fastSolution(selectionSize);
    System.out.println("finish eps-approx");

    // throw new ClassNotFoundException("Let us stop here");
  }

  public ProblemManager(Problem p, 
      Properties conf
      ) throws ClassNotFoundException {
    // TODO now only works for DTLS

    config = conf;

    maxEvaluations=  Integer.parseInt(config.getProperty("MaxEvaluations"));
    selectionSize = Integer.parseInt(config.getProperty("PopulationSize"));
    fileName = config.getProperty("filename");
    
    stEval = new Stack<>();
    
   // 1 10 100 1000 10000 100000
    
    ArrayList<Integer> list = new ArrayList<>();
    
    for(int i=100; i>=1; i--) {
    	list.add( (int) Math.pow(maxEvaluations, (i/100.0)));
    }
    
    list.add(200);
    list.add(1000);
    list.add(5000);
    list.add(10000);
    list.add(50000);
    list.add(100000);
    list.add(200000);
    
    Collections.sort(list);
    
    for(int n=list.size()-1; n>=0; n--) {
    	stEval.push(list.get(n));
    }


    hypName = fileName+ "-hyp";
    epsName = fileName+ "-eps";

    System.out.println("FileNames " + hypName +" " + epsName);
    try {
      String path =System.getProperty("user.home") + "/emoTests/" + hypName;

      System.out.println("PATH : " + hypName);
      hypWriter = new PrintWriter(new File(path));

      if (!(problem instanceof TTPProblem)) {
        path = System.getProperty("user.home") + "/emoTests/" + epsName;
        epsWriter = new PrintWriter(new File(path));
      }
    } catch (FileNotFoundException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    this.selectionSize = selectionSize;

    approx = new Point[sampleSize];

    problem = p;
    CalcApprox();
    sol = new ArrayList<Point>();


    numberOfVariables_  = p.getNumberOfVariables();
    numberOfObjectives_ = p.getNumberOfObjectives();
    numberOfConstraints_= p.getNumberOfConstraints();
    problemName_        = "ProblemManager";
    lowerLimit_ = new double[numberOfVariables_];
    upperLimit_ = new double[numberOfVariables_];  
    solutionType_ = new RealSolutionType(problem);



    if (problem instanceof DTLZ1 || problem instanceof DTLZ2 || problem instanceof DTLZ7 ||
        problem instanceof ZDT1 || problem instanceof ZDT2 || problem instanceof ZDT3 || problem instanceof ZDT6 ||
        problem instanceof WFG3 || problem instanceof WFG4 || problem instanceof WFG5 || problem instanceof WFG6 || problem instanceof WFG7

       ) {

      // do nothing

    } else if (problem instanceof TTPProblem)  {
      numberOfVariables_  = 2;
      numberOfObjectives_ = 2;
      numberOfConstraints_ = 1;

      // numberOfConstraints_= 0;
      problemName_        = "TTP";

      TTPProblem ttp = (TTPProblem) problem;
      solutionType_ = new TTPSolutionType(this, ttp.numCities, ttp.numItems);
      return ;
    } else if (problem instanceof ContinuousTurbineProblem){
      System.out.println("ARE WE HERE ?");
      problemName_ = "ContinuousTurbineProblem";
      // Solution Type
      solutionType_ = problem.getSolutionType();
      return;
    }
    else
    {
      throw new ClassNotFoundException("Problem is not in analyze");
    }

  
    initTime = 0; evalTime = analyzeTime = 0;

    // readFront();
      }

  @Override
  public double getLowerLimit(int i) {
    // TODO Auto-generated method stub
    return problem.getLowerLimit(i);
  }

  public Problem getSubProblem() {
    return problem;
  }

  @Override
  public double getUpperLimit(int i) {
    // TODO Auto-generated method stub
    return problem.getUpperLimit(i);
  }

  @Override
  public int getNumberOfObjectives() {
    // TODO Auto-generated method stub
    return problem.getNumberOfObjectives();
  }

  @Override
  public int getNumberOfVariables() {
    // TODO Auto-generated method stub
    return problem.getNumberOfVariables();
  }

  @Override
  public int getNumberOfConstraints() {
    // TODO Auto-generated method stub
    return problem.getNumberOfConstraints();
  }

  public Point[] convert(SolutionSet s) {
    Point[] p = new Point[s.size()];

    for(int n=0; n<s.size(); n++) {
      p[n] = new Point(s.get(n).getObjective(0), s.get(n).getObjective(1));
    }

    return p;
  }

  public Point[] getPointSet(){

    Point[] p = new Point[sol.size()];

    for(int n=0; n<sol.size(); n++) {
      p[n] = sol.get(n);
    }


    if (p.length >10) {
      Point[] ret = Eps.makeNonDominant(p);

      
      return ret;
    } else {
      return p;
    }

  }

  // returns the point set without deleting non-dominating points
  public Point[] getRawPointSet(){

    Point[] p = new Point[sol.size()];

    for(int n=0; n<sol.size(); n++) {
      p[n] = sol.get(n);
    }

    return p;
  }




  public void setLimit(int limit) {
    sol = new ArrayList<Point>();
  }

  double lower(double d) {
    return d-epsilonLength;
  }

  double upper(double d) {
    return d+epsilonLength;
  }

  public static void printSolution(Solution s) {
    System.out.println("Sol : " + s.getObjective(0) + " " + s.getObjective(1) );
  }

  public Point[] toPointSet(SolutionSet set) {
    ArrayList<Point> list = new ArrayList<Point>();

    for(int n=0; n<set.size(); n++) {
      double val1 = set.get(n).getObjective(0);
      double val2 = set.get(n).getObjective(1);

      if(set.get(n).getNumberOfViolatedConstraint() == 0 
          && !Double.isNaN(val1)
          && !Double.isNaN(val2)) {
        list.add(new Point(val1, val2));
          }
    }

    Point[] p = new Point[list.size()];

    for(int n=0; n<list.size(); n++) {
      p[n] = list.get(n);
    }

    return p;
  }


  public void analyzeTurbine(int evaluations, SolutionSet population) throws ClassNotFoundException {
		
      long overAllTime = System.currentTimeMillis() - initTime;

      if ( !stEval.empty() && stEval.peek() <= evaluations) {
      	
      	while(!stEval.empty() && stEval.peek() <= evaluations )
      	{
      		stEval.pop();
      	}
      	
      refX = 9;
      refY = 0.6;
      Point[] p = getPointSet();

      double maxX = 0;
      double maxY = 0;

      for (Point a : p) {
        if (!Double.isNaN(a.y) && !Double.isNaN(a.x)) {
          maxX = Math.max(maxX, a.x);
          maxY = Math.max(maxY, a.y);
        }
      }

      // System.out.println("Turbine called X: "+ maxX + " Y: "+ maxY);

      Point[] populationPoints = toPointSet(population);


      // TODO get rid of not 
      double before =   Algo.hyperCalc2(populationPoints, population.size(), refX, refY );
      
      long hypTimeBefore = System.currentTimeMillis();
      double after =   Algo.hyperCalc2(getPointSet(), population.size(), refX, refY);
      long hypTime = System.currentTimeMillis() - hypTimeBefore;


      String val = 
        evaluations  +  " "+ // 1
        before+ " " + // 2
        after + " " + // 3
        (before-after) + " " +  // 4
        getPointSet().length + " " + // 5
       Eps.makeNonDominant(toPointSet(population)).length + " " + // 6
        analyzeTime + " " + // 7
        evalTime + " " + // 8
        overAllTime  + " " + // 9
        hypTime; // 10


      System.out.println("hyp : " + val);
      hypWriter.println(val);
    }
  }


  public void analyzeTTP(int evaluations, SolutionSet population) throws ClassNotFoundException {
    {
      int k = evaluations;
      int i =evaluations %(100 * 100);

      evaluations /= 100;
    


      if ( i==0) { //getPointSet().length >= selectionSize && (!stEval.empty()  && evaluations >= stEval.peek())) {
        while (!stEval.empty()  && evaluations >= stEval.peek()) {
          stEval.pop();
        }

        // if (getPointSet().length >= selectionSize) {
        refX   = 30000;
        refY = 1000;
        ArrayList<Point> list = new ArrayList<Point>();

        for(int n=0; n<population.size(); n++) {
          if(population.get(n).getNumberOfViolatedConstraint() == 0) {
            list.add(new Point(population.get(n).getObjective(0), population.get(n).getObjective(1)));
          }
        }

        Point[] p = new Point[list.size()];

        for(int n=0; n<list.size(); n++) {
          p[n] = list.get(n);
        }
        double hypMax = 0; //2.8189230882325925E7;

        // TODO get rid of not 
        double before =   Algo.hyperCalc2(p, population.size(), refX, refY );
        double after =   Algo.hyperCalc2(getPointSet(), population.size(), refX, refY);


        //printSolution(population.get(0));
        //printSolution(population.get(selectionSize-1));

        double maxX= 0;
        double maxY = 0;

        for (Point p1 : getPointSet()) {
          maxX = Math.max(maxX, p1.x);
          maxY = Math.max(maxY, p1.y);
        }

        String val = evaluations +  " " +(after - before) + " " + before + " " + after + " max :" +maxX+" "+maxY + "numberNOnDominating : "+ getPointSet().length;

        // String val = evaluations +  " " + before + " " + after + " "+ getPointSet().length;

        System.out.println("hyp : " + val);
        hypWriter.println(val);
        //	}
      }}
    }

    public void analyzeBenchmarks(int evaluations, SolutionSet population) throws ClassNotFoundException, InvalidAlgorithmParameterException {
        long overAllTime = System.currentTimeMillis() - initTime;

        if (config.getProperty("feeding","no").equals("yes")) {
	        if (evaluations % (100*Integer.parseInt(config.getProperty("feedRate"))) == 0) {
	        Point[] replacement = Algo.hyperCalcPoints(getPointSet(), population.size(), refX, refY);
	
	        for(int n=0; n<replacement.length; n++) {
	        	population.get(n).setObjective(0, replacement[n].x);
	        	population.get(n).setObjective(1, replacement[n].y);
	        }
	        }
        }
    	
        if ( !stEval.empty() && stEval.peek() <= evaluations) {
        	
        	while(!stEval.empty() && stEval.peek() <= evaluations )
        	{
        		stEval.pop();
        	}



          double before =  Algo.hyperCalc2(Eps.makeNonDominant(toPointSet(population)), population.size(), refX, refY);
          long hypTimeBefore =  System.currentTimeMillis();
          double after =   Algo.hyperCalc2(Eps.makeNonDominant(getPointSet()), population.size(), refX, refY);
          long hypTime = System.currentTimeMillis()- hypTimeBefore;

          // System.out.println("Before: " + evaluations + " " + before + " " + after);

          before =  optHyp - before;
          after =  optHyp - after;

          double beforeRelative = before / optHyp;
          double afterRelative = after / optHyp;

          int generationen = evaluations;


          String val = 
            generationen  +  " "+ // 1
            before+ " " + // 2
            after + " " + // 3
            (before-after) + " " +  // 4
            beforeRelative + " " + // 5
            afterRelative + " " + // 6
            getPointSet().length + " " + // 7
            Eps.makeNonDominant(toPointSet(population)).length + " " + // 8
            analyzeTime + " " + // 9
            evalTime + " " + // 10
            overAllTime + " " + // 11
            hypTime; // 12

          System.out.println("hyp : " + val);

          hypWriter.println(val);
        


        long epsTimeBefore =  System.currentTimeMillis();
        Point[] p = Eps.makeNonDominant(getPointSet());
        Eps eps = new Eps(p, p);
        double res = eps.fastSolution(selectionSize);
        Point[] postPoints = Eps.makeNonDominant(eps.epsilonPoints(res, selectionSize));

        long epsTime = System.currentTimeMillis()- epsTimeBefore;

        Point[] populationPoints = Eps.makeNonDominant(convert(population));
        eps = new Eps(approx, postPoints);
        double post = eps.fastSolution(selectionSize);
        
        
        eps = new Eps(approx, populationPoints);
        double pre = eps.fastSolution(selectionSize);

         before =  upper(pre) - lower(optEps);
         after =  upper(post) - lower(optEps);

          val =  
                 generationen  +  " " + // 1
                 before+ " " + // 2
                 after + " " + // 3
                 (before-after) + " " +  // 4
                 getPointSet().length + " "+ // 5
                 Eps.makeNonDominant(toPointSet(population)).length + " " + // 6
                 analyzeTime + " " + // 7
                 evalTime + " " + // 8
                 overAllTime + " " + // 9
                 epsTime; // 10*/;

        System.out.println("eps : " + val);

        epsWriter.println(val);

        }
        
        
    }


    public void analyzeProblem(int evaluations, SolutionSet population) throws ClassNotFoundException, InvalidAlgorithmParameterException {

    	evaluations += evalRound;
      lastPopulation = population;

      if (config.getProperty("analyze", "yes").equals("no")) {
        if (evaluations %1000 == 0) {
          System.out.println("num :" + evaluations);
        }
        return;
      }

      long beforeAnalyze =  System.currentTimeMillis();

      if (problem instanceof ContinuousTurbineProblem) {
        analyzeTurbine(evaluations, population);
      } else if (problem instanceof TTPProblem) {
        analyzeTTP(evaluations, population);
      } else {
        analyzeBenchmarks(evaluations, population);
      }


      // add time for analyzing
      analyzeTime += System.currentTimeMillis() - beforeAnalyze;

    }

    public void clearSolutionSet() {
      sol.clear();
    }

    public int getSelectionSize() {
      return selectionSize;
    }

    public int getTotalGenerationSize() {
      return maxEvaluations/selectionSize;
    }

    public void closeAndWriteGNUPLOT() {
      hypWriter.close();
      epsWriter.close();
      
      System.out.println("fileName :" + fileName + "-hyp");

      if(config.getProperty("bestof", "no").equals("yes")) {
      String path =System.getProperty("user.home") + "/emoTests/" + hypName +".gnu";

      GNUPrinter.print(this,true, problem instanceof TTPProblem, hypName, hypName, path);

      if (!(problem instanceof TTPProblem)){
        path =System.getProperty("user.home") + "/emoTests/" + epsName +".gnu";

        GNUPrinter.print(this,false, false, epsName, epsName, path);	
      }

      String PATH =System.getProperty("user.home") + "/vermutung/wfg-plot1";

      System.out.println("PATH : " + PATH);
      try {
        PrintWriter writer = new PrintWriter(new File(PATH));

        System.out.println("last population");
        for (int i=0; i<lastPopulation.size(); i++) {
          writer.println(lastPopulation.get(i).getObjective(0) + " " + lastPopulation.get(i).getObjective(1));
        }
        writer.close();
      } catch (FileNotFoundException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

      PATH =System.getProperty("user.home") + "/vermutung/wfg-plot2";

      System.out.println("PATH : " + PATH);
      try {
        PrintWriter writer = new PrintWriter(new File(PATH));

        System.out.println("all the points");
        Point[] p = getPointSet();
        for (int i=0; i<p.length; i++) {
          writer.println( p[i].x+ " " + p[i].y);
        }
        writer.close();
      } catch (FileNotFoundException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      }



    }

    @Override
    public void evaluate(Solution solution) throws JMException {
    	if (initTime == 0) {
    		initTime = System.currentTimeMillis();
    	}
      long beforeTime = System.currentTimeMillis();
      problem.evaluate(solution);	
      evalTime += System.currentTimeMillis() - beforeTime;
    }

    @Override
    public void evaluateConstraints(Solution solution) throws JMException {
      long beforeTime = System.currentTimeMillis();
      problem.evaluateConstraints(solution);
      evalTime += System.currentTimeMillis() - beforeTime;
      // only add Solutions that are not vialoted

      double val1 = solution.getObjective(0);
      double val2 = solution.getObjective(1);

      if (solution.getNumberOfViolatedConstraint() == 0 
          && !Double.isNaN(val1) 
          && !Double.isNaN(val2)) {
        //	System.out.println("yes");

        sol.add(new Point(val1, val2));
          } 
    }

    public double getOptHyp() {
      return optHyp;
    }
    
    int evalRound = 0;

	public void addEvaluationRound() {
		evalRound += maxEvaluations;
	}

  }
