package HyperEps;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;

import jmetal.core.Problem;
import jmetal.core.Solution;
import jmetal.core.Variable;
import jmetal.encodings.solutionType.BinaryRealSolutionType;
import jmetal.encodings.solutionType.RealSolutionType;
import jmetal.encodings.variable.Binary;
import jmetal.encodings.variable.Permutation;
import jmetal.util.JMException;


public class TTPProblem extends Problem {
	
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
/*

	public static void main(String[] args) {
		
    	String path = System.getProperty("user.home") +"/hypselection/ttp/";
    	if (args.length == 1) {
    	path = path + "a280-10.ttp";
    	} else {
    		path = path + args[1];
    	}
    	Problem problem =new TTPProblem(path);
        
    	try {
			Solution newSolution = new Solution(problem);
			problem.evaluate(newSolution);
			System.out.println("Objectives : " + newSolution.getObjective(0) + " " + newSolution.getObjective(1));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 

    	
    }
  */  
    int numCities;
    int numItems;
    long capKnapsack;
    double minSpeed;
    double maxSpeed;
    double R;
    
    double dropRate = 0.9;
    double C = 1;
    
    City[] city;
    Item[] item;
    
    
private String read(String line) {
	 line = line.substring(line.indexOf(":")+1);
     return line.replaceAll("\\s+","");
}
    
	
 public TTPProblem(String path) {
	 
	 System.out.println("Start reading file");
	 this.readFile(path);
	 System.out.println("Finished reading file");
	 
	 numberOfVariables_  = 2;
	 numberOfObjectives_ = 2;
	 numberOfConstraints_ = 1;
	 
   // numberOfConstraints_= 0;
	 String[] split = path.split("/");
	 
   problemName_        = "TTP-"+ split[split.length-1];
   
   solutionType_ = new TTPSolutionType(this, numCities, numItems);     
 } //DTLZ2
 

private void readFile(String fileName) {
	File file = new File(fileName);
	
	   BufferedReader br = null;
       try {
           br = new BufferedReader(new FileReader(file));

           String line;
           while ((line = br.readLine()) != null) {
               // process the line

           
               if (line.startsWith("DIMENSION")) {
                   this.numCities = Integer.parseInt(read(line));
               } else if (line.startsWith("NUMBER OF ITEMS")) {
            	   this.numItems = Integer.parseInt(read(line));
               } else if (line.startsWith("CAPACITY OF KNAPSACK")) {
                   this.capKnapsack =Long.parseLong(read(line));
               } else if (line.startsWith("MIN SPEED")) {
                   this.minSpeed=Double.parseDouble(read(line));
               } else if (line.startsWith("MAX SPEED")) {;
                   this.maxSpeed=Double.parseDouble(read(line));
               }
               else if (line.startsWith("RENTING RATIO")) {
                   this.R = Double.parseDouble(read(line));
               } else if (line.startsWith("NODE_COORD_SECTION")) {
            	   this.city = new City[this.numCities];
            	   
                   for (int i=0; i<this.numCities; i++) {
                       String [] words = br.readLine().split("\\s+");
                       
                       
                       this.city[i] = new City(Double.parseDouble(words[1])
                    		   , Double.parseDouble(words[2]));
                   }
               }
               if (line.startsWith("ITEMS SECTION")) {
                   this.item = new Item[numItems];
                   for(int i=0; i<this.numItems; i++) {
                	   line = br.readLine();
                       String[] splittedLine = line.split("\\s+");
                       item[i] = new Item(
                    		   i,
                    		   Integer.parseInt(splittedLine[1]),
                    		   Integer.parseInt(splittedLine[2])
                    		   );
                       
                       int cityPos = Integer.parseInt(splittedLine[3]) -1;
                       //System.out.println(line +"items :" + numItems + " " + numCities + " " +i + " " +cityPos);
                       city[cityPos].addItem(item[i]);;
                   }
               }
           }
           br.close();
       } catch (IOException ex) {
    	   ex.printStackTrace();
       }
}

// test the the sum is not bigger than the capacities
@Override
public void evaluateConstraints(Solution solution) throws JMException {
	// TODO Auto-generated method stub
	super.evaluateConstraints(solution);
	
	Variable[] vars = solution.getDecisionVariables();
	Binary binary = (Binary) vars[1];
	long total = 0;
	
	int count = 0;
	
	for(int i=0; i<numItems; i++) {
		if(binary.getIth(i) ) {
			total += item[i].getWeight();
			count++;
		}
	}
	
	// System.out.println("count " + count);
	
	double diff = total - capKnapsack;
	
	if (diff > 0) {
	    solution.setOverallConstraintViolation(-diff);
	    solution.setNumberOfViolatedConstraint(1);
	} else {
		solution.setNumberOfViolatedConstraint(0);
	}
}

	public double calcDiff(Solution solution) {
		Variable[] vars = solution.getDecisionVariables();
		Binary binary = (Binary) vars[1];
		long total = 0;
		
		int count = 0;
		
		for(int i=0; i<numItems; i++) {
			if(binary.getIth(i) ) {
				total += item[i].getWeight();
				count++;
			}
		}
		
		// System.out.println("count " + count + " total " + total);
		
		double diff = total - capKnapsack;
		return diff;
	}

	@Override
	public void evaluate(Solution solution) throws JMException {
		
		
		Variable[] vars = solution.getDecisionVariables();
		
		Permutation p = (Permutation) vars[0];
		Binary binary = (Binary) vars[1];
		
		int[] perm = p.vector_;
		
		City currentCity = this.city[perm[0]];
		long currentWeight = 0;
		double speed = maxSpeed;
		
		double totalTime =0;
		
		ArrayList<Item> taken = new  ArrayList<Item>();
		
		for(int n=1; n<= this.numCities; n++) {
			// get the items
			for (Item i : currentCity.getItems()) {
				if (binary.getIth(i.pos)){
					currentWeight += i.getWeight();
					i.setTakenTime(totalTime);
					taken.add(i);
				}
			}
			
			speed = maxSpeed + currentWeight * (maxSpeed - minSpeed) / (capKnapsack);

			City nextCity = city[perm[n %  this.numCities]];
			totalTime += currentCity.calcDistance(nextCity) / speed;
			currentCity = nextCity;
		}
		
		double totalProfit = 0;
		
		for(Item item : taken) {
			double timeDiff = totalTime - item.takenTime;
			double fac =  Math.pow(dropRate, Math.floor(timeDiff/C));
			totalProfit += item.getProfit() * fac;
		}
		
		double upperbound = 0;
		for(Item i : item) {
			upperbound = Math.max(upperbound, i.getProfit());
		}
		
		upperbound *= capKnapsack;
		
	//	System.out.println("UPPER " + upperbound);
		
	//	System.out.println("DIFF :" + calcDiff(solution));
		
		solution.setObjective(0, totalTime);
		solution.setObjective(1, 1000- totalProfit);
		
		
	//	System.out.println("UPPER " + upperbound + " " + solution.getObjective(0) + " " + solution.getObjective(1));
	}

}
