#include <iostream>
#include <stdio.h>
#include <vector>
#include <stack>
#include <ctime>
#include <math.h>
#include <cstring>
#include <stdlib.h>

using namespace std;

struct Point{
  double x;
  double y;
};

struct Function {
  // f = m * x + n
  double m;
  double n;
  int number;
};

double Calc(Function f, int pos) {
  return f.m * pos + f.n;
}


// the intersection point from two functions f1 and f2 with different slopes
double IntersectX(Function f1, Function f2) {
 return (f2.n - f1.n) / (f1.m - f2.m);
}

class Solver {
  public: 
    Solver() {
      init();
    }

    void init() {
      scanf("%d", &N);

      // start reading
      points = vector<Point>(N);
      fun = vector<Function>(N);

      for(int n=0; n < N; n++) {
        double x, y;
        scanf("%lf %lf", &x, &y);
        points[n].x = x;
        points[n].y = y;

        fun[n].m = -y;
        fun[n].n = x * y;
        fun[n].number = n;
      }
#ifdef DEBUG
      cout<<"end reading"<<endl;
#endif
    }

    void PrintAll() {
      int counter = 0;
#ifdef DEBUG
      for(Function f : fun) {
        cout<<"i : "<<counter++<<" : "<<Calc(f, 0) <<endl;
      }
#endif
    }

    double Solve1() {
      double best = -1;
      int counter = 0;
      for(vector<Function>::iterator it =fun.begin(); it!=fun.end(); it++) {
        best = max(best, Calc(*it,0));
      }

      return best;
      // cout<<" Solve 1 : "<<best<<endl;
    }

    void Solve2() {
      double best = -1;

      for(int n=0; n < N; n++) {
        cout<<"N: "<<n<<"of "<<N<<endl;
        for(int m=n+1; m < N; m++) {
          double val = Calc(fun[n],0) + Calc(fun[m], points[n].x);
#ifdef DEBUG
          cout<<"("<<n<<","<<m<<") : "<<val<<endl;
#endif
          best = max(best, val);
        }
      }

      cout<<"Solve 2 : " << best <<endl;
    }

    void Solve3() {
      double best = -1;

      for(int n=0; n < N; n++) {
        for(int m=n+1; m < N; m++) {
          for (int k=m+1; k < N; k++) {
            double val = Calc(fun[n],0) + Calc(fun[m], points[n].x) + Calc(fun[k], points[m].x);

//            cout<<"("<<n<<","<<m<<","<<k<<") : "<<val<<endl;
            best = max(best, val);
          }
        }
      }

      cout<<"Solve 3 : " << best <<endl;

    }

    void Faster2() {

      vector<double> best(N);
      vector<Function> st;

      st.push_back(fun[N-1]);

      // add all the functions to the stack
      // that are relevant
      for (int i = N-2; i > 0; i--) {
        if(Calc(fun[i],0) >= Calc(st[st.size()-1], 0)) {
          st.push_back(fun[i]);
        }
      }
      cout<<"SIZE : "<<st.size()<<endl;

      // evaluate for the points
      for(int n=0; n<N-1; n++) {
        // look whether we are still at the maximum function
        while(st.size() >1 && 
          (Calc(st[st.size()-1], points[n].x ) < Calc(st[st.size()-2], points[n].x ) ||
          st[st.size()-1].number < n + 1
          
          )) {
          st.pop_back();
        }

        best[n] = Calc(st[st.size()-1], points[n].x);
#ifdef DEBUG
        cout<<"n : " <<n<<" best "<<st[st.size()-1].number <<" :  "<<best[n]<<endl;
#endif
      }

      double res = -1;

      for(int n=0; n < N-1; n++) {
          double val = Calc(fun[n],0) + best[n];
#ifdef DEBGUG
          cout<<"("<<n<<") : "<<val<<endl;
#endif
          res = max(res, val);
      }

      cout<<"Faster 2 : "<<res<<endl;
    }

#define EPSILON  1e-8

    vector<double> CalcMaxValues2(vector<Function> &f) {
      vector<double> best(N);
      vector<Function> s(N);

      int len = 0;
      int left = 0;
      int right = -1;

      for (int i = N - 2; i >= 0; i--) {

        while (len > 1 ) {

          double d1 = IntersectX(f[i+1], s[right]);
          double d2 = IntersectX(s[right], s[right - 1]);

          if (d1 > d2 || fabs(d1-d2) < EPSILON) {
            len--;
            right--;
          } else {
            break;
          }
        }

        s[right+1] = f[i+1];
        right++;
        len++;


        while (len > 1) {
          double d1 = Calc(s[left], points[i].x);
          double d2 = Calc(s[left + 1], points[i].x);

          if (d1 < d2 || fabs(d1-d2) < EPSILON) {
            len--;
            left++;
          } else {
            break;
          }
        }

        best[i] = Calc(s[left], points[i].x);
      }

      return best;
    }

    // given function with decreasing slope
    // find out for the n positions 
    // what is the maximum value
    // in linear time
    vector<double> CalcMaxValues(vector<Function> &f) {
      vector<double> best(N);
      vector<Function> st;

      st.push_back(f[N-1]);

      // add all the functions to the stack
      // that are relevant
      for (int i = N-2; i > 0; i--) {
        if(Calc(f[i],0) >= Calc(st[st.size()-1], 0)) {
          st.push_back(f[i]);
        }
      }
      // cout<<"SIZE : "<<st.size()<<endl;

      // evaluate for the points
      for(int n=0; n<N-1; n++) {

  //      cout<<" n : "<<n<<endl;
        // look whether we are still at the maximum function
        if (n == 0 ){
  //        cout<<"HELLO "<<st[st.size()-1].m<<" "<<st[st.size()-1].n<<endl;
  //        cout<<"HELLO "<<st[st.size()-2].m<<" "<<st[st.size()-2].n<<endl;
          double val1 = Calc(st[st.size()-1], points[n].x );
          double val2 =  Calc(st[st.size()-2], points[n].x );

//          cout<<val1<<" "<<val2<<endl;
        }
        while(st.size() >1 && 
            (Calc(st[st.size()-1], points[n].x ) < Calc(st[st.size()-2], points[n].x ) ||
             st[st.size()-1].number < n + 1

            )) {
          // cout<<"POP"<<endl;
          st.pop_back();
        }

        // cout<<"VAL : "<<st[st.size()-1].m <<" "<<st[st.size()-1].n<<endl;
        best[n] = Calc(st[st.size()-1], points[n].x);
        
      //   cout<<best[n]<<endl;
      }

      return best;
    }




    double  Faster(int K) {

      if (K==1) {
        return Solve1();
      }


      vector<double> best = CalcMaxValues2(fun);
/*      vector<double> best2 = CalcMaxValues2(fun);



for(int n=best.size()-1; n>=0 ; n--) {
        if (fabs(best[n]- best2[n]) > 0.001) {

          cout<<"ERROR " <<endl;

          cout<<"N : "<<n<<" "<<best[n]<<" "<<best2[n]<<endl;
          cout<<endl;

          int pos = 0;

          double res = 0;
          for (int m=n+1; m<N;m++) {
            if ( res <  Calc(fun[m], points[m].x)) {
              pos = m;
            }
            res = max(res, Calc(fun[m], points[m].x));
          }

          cout<<"res: "<<res<<" pos: "<<pos<<endl;



          cout<<endl<<endl;

          return 0;
        }
     }
*/

      double res = -1;

      for(int k=0; k<K-2; k++) {
        vector<Function> F(N);

        for(int n=0; n < N; n ++) {
          F[n].m = fun[n].m;
          F[n].n = fun[n].n + best[n];
          F[n].number = fun[n].number;
        }

        best = CalcMaxValues2(F);
       // best2 = CalcMaxValues2(F);

/*for(int n=0; n<best.size(); n++) {
        if (fabs(best[n]- best2[n]) > 0.001) {

          cout<<"ERROR " <<endl;

          cout<<"N : "<<n<<" "<<best[n]<<" "<<best2[n]<<endl;
          cout<<endl;

          double res = 0;
          for (int m=n+1; m<N;m++) {
            res = max(res, Calc(F[m], points[n].x));
          }

          cout<<"res: "<<res<<endl;



          cout<<endl<<endl;

        }
     }*/


      }

    //  for(int n=0; n<N; n++) {
    //    cout<<"BEST : "<<n<<" : "<<best[n]<<endl;
    //  }


      for(int n=0; n<N;n++) {
 //       cout<<" VAL : "<<n<<" "<< Calc(fun[n],0)+best[n]<<endl;
        res = max(res, Calc(fun[n],0) + best[n]);
      }

      return res;
      // cout<<"Faster (N*K)        : "<<res<<endl;
    }

    double DPSolution(int K) {
      if (K <=0) {
        cout<<"DPSolution " <<" wrong output"<<endl;
      }

      vector<vector<double> > dp(N, vector<double>(K+1, - 1));

      double res = Rec(0, K, dp);

      return res;

     // cout<<"DP Solution (N*N*K) : "<<res<<endl;
    }

    double Rec(int pos, int K, vector<vector<double> > & dp) {
#ifdef DEBUG
      cout<<"pos "<<pos<<" K : "<<K<<endl;
#endif

      if(K==0) {
        return 0;
      } else if (pos >= N) {
        // TODO is this sufficient?
        return -100;
      } else {
        if(dp[pos][K] == -1) {
          double val = 0;

          for(int n=pos; n < N; n++){
            double plus = Calc(fun[n],(pos==0? 0 :  points[pos-1].x));
            val = max(
              val, 
              plus + Rec(n+1, K-1, dp)
            );
          }

          dp[pos][K] = val;
        }

        return dp[pos][K];
      }
    }

    int getLength() {
      return N;
    }

  private:
    vector<Point> points;
    vector<Function> fun;
    int N;
};

#define DIVIDE 1000

int main(int argc, char* argv[]) {

  if (argc < 2) { // first argument k
    cout<<"USAGE: ./a.out 'k' [--timeMode] [--dp] [--fast] <INPUT_STREAM"<<endl;
    return 0;
  } 

  int k = atoi(argv[1]);

  bool calcDP = false;
  bool calcFast = false;
  bool half = false;
  bool timeMode = false;

  for (int n=2; n<argc; n++) {
    if(strcmp(argv[n], "--dp") == 0) {
      calcDP = true;
    } else if (strcmp(argv[n], "--fast") == 0) {
      calcFast = true;
    } else if (strcmp(argv[n], "--half") == 0) {
      half = true;
    } else if (strcmp(argv[n], "--timeMode") == 0) {
      timeMode = true;
    }
    else {
      cout<<" UNKNOWN PARAMETER " <<argv[n]<<endl;
    } 
  }

  Solver* sol = new Solver();
  if (sol->getLength()< k) {
    return 0;
    cout<<" K is bigger than N !"<<endl;
  }

  if (half) {
    k = sol->getLength() / 2;
  }

  if (calcFast) {

    std::clock_t start = std::clock();

    double res = sol->Faster(k);

    if (timeMode) {
      cout<<sol->getLength() << " "<< (std::clock() - start) / (double)(CLOCKS_PER_SEC / DIVIDE)<<endl;
    } else {
      cout<<"FAST SOLUTION : "<<res << endl;
    }
  } 

  if (calcDP) {

    std::clock_t start = std::clock();

    double res = sol->DPSolution(k);
    
    if (timeMode) {
      cout<<sol->getLength() << " "<< (std::clock() - start) / (double)(CLOCKS_PER_SEC / DIVIDE) << endl;
    } else {
      cout<<"DP SOLUTION   : " << res << endl;
    }
  }
}

